// -These are all libraries used by the entire code. 
// -Each file needs to include their own required libraries/files and *only* those, not more
#include <iostream>
#include <vector>
#include <string>
using namespace std; 

// -When there is a class dependency cycle, a forward reference needs to be used.
// -This tells the compiler that the id "Player" represents a class.
// -In order for the class to be used, the entire class needs to be properly defined somewhere, 
//    enabling the compiler/linker to process code that requires: 
//    1. to know the size of objects of this kind. 
//    2. to know what are the members of that class. 
// -Unless either of these two is required, a forward declaration is sufficient. 
class Player; 

class Country {
  public: 
	Player *owner;
	static int numRegisteredCountries;
	Country() : owner(NULL), name("") { numRegisteredCountries++; }
	Country(Player* owner) : owner(owner) { 
		numRegisteredCountries++; 
		name = "country" + to_string(numRegisteredCountries);  }
	string getName() {
		return name; 
	}
  private:
	string name; 
	// note that a friend is not a member
	friend ostream& operator<<(ostream& os, Country& co);
};

class Player {
  public:
	vector<Country*> ownedCountries; 
	static int numRegisteredPlayers; 
	Player() { 
		numRegisteredPlayers++;
		name = "player" + to_string(numRegisteredPlayers); 
	};
	string getName() {
		return name;
	}
private:
	string name;
	friend ostream& operator<<(ostream& os, Player& pl);
};

// -These are declared/defined in the global scope. 
// -When separating the code, the question is: in which file should these declarations/definitions be? 
// -If they are put in header files, that will mean that this header file can only be included once in 
//    an *entire application*, since the whole application shares their global scope. 
const int numPlayers = 4; 
const int numCountries = 13; 
// -These generally need to be included in their corresponding cpp file. 
// -If they would be put in the header file, that would mean that this header file cannot be included by 
//    more than one compilation unit, or else it would be redefined. 
int Player::numRegisteredPlayers = 0;
int Country::numRegisteredCountries = 0; 

// -This is an operator declared as a fiend of the Country class. 
// -As such, it needs to be defined in the cpp file corresponding to the header
//    file where the Country class is declared, or else the cpp file in which it is defined 
//    would need to be compiled in order for this operator to be available for 
//    linkage. 
ostream& operator<<(ostream& os, Country& co){
	os << co.name << " : owner is " << co.owner->getName() << endl;
	return os;
}

// -This is an operator declared as a fiend of the Player class. 
// -As such, it needs to be defined in the cpp file corresponding to the header
//    file where the Player class is declared, or else the cpp file in which it is defined 
//    would need to be compiled in orser for this operator to be available for 
//    linkage. 
ostream& operator<<(ostream& os, Player& pl){
	os << pl.name << " : owned countries are : " << endl;
	for (Country* c : pl.ownedCountries)
		os << *c; 
	cout << endl;
	return os;
}

int main() {
	vector<Player*> allPlayers; 
	vector<Country*> allCountries; 
	for (int i = 1; i <= numPlayers; i++) 
		allPlayers.push_back(new Player());
	for (int j = 0; j <= numCountries -1 ; j++) {
		allPlayers.at(j % numPlayers)->ownedCountries.push_back(new Country(allPlayers.at(j % numPlayers)));
	}
	for (int k = 0; k <= numPlayers - 1; k++)
		cout << *allPlayers.at(k);
}
